@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 
 import json
+import bjson
 import logging
 import random
 import socket
@@ -47,11 +48,10 @@ class Core(Thread):
         self.f_stop = True
         self.traffic.close()
         # Server/client のソケットを全て閉じる
+        for user in self.user:
+            self.remove_connection(user, 'Manually close.')
         try: self.server_sock.close()
         except: pass
-        for user in self.user:
-            try: user.sock.close()
-            except: pass
 
     def run(self):
         self.traffic.start()
@@ -67,8 +67,7 @@ class Core(Thread):
         while not self.f_stop:
             try:
                 sock, host_port = server_sock.accept()
-                Thread(target=self.__initial_connection_check,
-                       args=(sock, host_port), daemon=True).start()
+                Thread(target=self.__initial_connection_check, args=(sock, host_port), daemon=True).start()
 
             except json.decoder.JSONDecodeError:
                 try: sock.close()
@@ -150,7 +149,7 @@ class Core(Thread):
             self.traffic.put_traffic_up(encrypted)
 
             Thread(target=self.__receive_msg,
-                   name='C:' + new_user.name, daemon=True, args=(new_user,)).start()
+                   name='C:' + new_user.name, args=(new_user,), daemon=True).start()
 
             c = 20
             while len(self.user) == 0 and c > 0:
@@ -174,13 +173,13 @@ class Core(Thread):
             logging.error("NewConnectionError {} {}".format(host_port, e), exc_info=Debug.P_EXCEPTION)
         return False
 
-    def remove_connection(self, user):
+    def remove_connection(self, user, reason=None):
         with self.lock:
             if user in self.user:
                 self.user.remove(user)
                 try: user.close()
                 except: pass
-                logging.debug("remove connection to %s" % user.name)
+                logging.debug("remove connection to {} by \"{}\"".format(user.name, reason))
                 return True
             return False
 
@@ -191,8 +190,10 @@ class Core(Thread):
         if len(self.user) == 0:
             raise ConnectionError('client connection is zero.')
         elif len(msg_body) > C.MAX_RECEIVE_SIZE + 5000:
-            raise ConnectionRefusedError('Max message size is {}Kb (You try {}Kb)'
-                                         .format(C.MAX_RECEIVE_SIZE / 1000, len(msg_body) / 1000))
+            error = 'Max message size is {}kb (You try {}Kb)'.format(
+                round(C.MAX_RECEIVE_SIZE/1000000, 3), round(len(msg_body)/1000000, 3))
+            self.send_msg_body(msg_body=bjson.dumps(error), user=user)
+            raise ConnectionRefusedError(error)
         elif user is None:
             user = random.choice(self.user)
 
@@ -254,8 +255,8 @@ class Core(Thread):
         except Exception as e:
             logging.debug(e, exc_info=Debug.P_EXCEPTION)
         else:
-            Thread(target=self.__receive_msg, name='S:' + new_user.name,
-                   daemon=True, args=(new_user,)).start()
+            Thread(target=self.__receive_msg,
+                   name='S:'+new_user.name, args=(new_user,), daemon=True).start()
             return
         # close socket
         try: sock.close()
@@ -285,7 +286,7 @@ class Core(Thread):
                 msg_body = first_msg[4:]
 
                 # Notice long message
-                if msg_len != len(msg_body):
+                if Debug.F_LONG_MSG_INFO and msg_len != len(msg_body):
                     logging.debug("Receive long msg, len=%d, body=%d" % (msg_len, len(msg_body)))
 
                 if msg_len == 0:
